use std::{
    thread,
    time::{Duration, Instant},
};

use bitvec::{order::Msb0, view::AsBits};
use colored::Colorize;
use rapidriter_cat::next_frame;

const WIDTH: usize = 96;
const HEIGHT: usize = 38;

fn main() {
    // clear screen
    print!("\x1B[2J");

    let mut frame = [0; 456];

    let start = Instant::now();
    for i in 0..100u16 {
        let target = start + Duration::from_millis(u64::from(i) * 100);
        let now = Instant::now();
        if now < target {
            thread::sleep(target - now);
        }

        if next_frame(i, frame.as_mut_ptr()) == 1 {
            break;
        }

        draw_frame(&frame);
    }
}

fn draw_frame(frame: &[u8; 456]) {
    // reset cursor position to (1,1)
    print!("\x1B[1;1H");

    let bv = frame.as_bits::<Msb0>();

    // top part of border
    for _ in 0..(WIDTH + 2) {
        print!("{}", " ".on_white());
    }
    println!();

    for y in 0..HEIGHT {
        print!("{}", " ".on_white());

        for x in 0..WIDTH {
            if bv[x + y * WIDTH] {
                print!("{}", " ".on_red());
            } else {
                print!(" ");
            }
        }

        print!("{}", " ".on_white());

        println!();
    }

    // bottom part of border
    for _ in 0..(WIDTH + 2) {
        print!("{}", " ".on_white());
    }

    println!();
}
